import pandas as pd
import os


def get_dictionary():
    """
    Gets a dictionary of all items in Amsterdam UMCdb, with (work in progress) translated
    medical concepts.

    Returns:
        dataframe:
    """
    dirname = os.path.dirname(os.path.abspath(__file__))
    filename = './dictionary/dictionary.csv'

    dictfile = os.path.join(dirname, filename)

    dictionary = pd.read_csv(dictfile)
    return dictionary



"""amsterdamumcdb
Contains functions for simplifying data analysis and cleanup of the AmsterdamUMCdb database
"""

# For development:
# pip install -e ~/AmsterdamUMCdb/

#from PiPy repository (released version)
# pip install amsterdamumcdb





import pandas as pd
import numpy as np
import os


def get_fluidbalance(admissionid, from_date, to_date, con) -> pd.DataFrame:
    """Calculate the fluid balance (i.e. fluid input - fluid output) for a specific time interval.
    Typically this will be used to calculate daily fluid balances.

    Arguments:
        admissionid -- the admissionid of ICU admission
        from_date -- the start of the interval, expressed as the number of milliseconds from start of this ICU admission
        to_date -- the end of the interval, expressed as the number of milliseconds from start of this ICU admission
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """

    try:
        admissionid = np.int64(admissionid)
    except ValueError:
        raise Exception("admissionid is not a number: %r" % admissionid)

    try:
        from_date = np.int64(from_date)
    except ValueError:
        raise Exception("from_date is not a number: %r" % from_date)

    try:
        to_date = np.int64(to_date)
    except ValueError:
        raise Exception("to_date is not a number: %r" % to_date)

    # gets the SQL source file
    dirname = os.path.dirname(os.path.abspath(__file__))
    filename = './sql/functions/get_fluidbalance.sql'
    sqlfile = os.path.join(dirname, filename)

    file = open(sqlfile, 'r')

    sql = file.read().format(admissionid=admissionid, msfromdate=from_date, mstodate=to_date)

    if type(con) is dict:  # BigQuery config
        result = pd.read_gbq(sql, configuration=con)
    else:
        result = pd.read_sql(sql, con)

    return result



import matplotlib.pyplot as plt
import numpy as np

def outliers_histogram(data, z_treshold=4.0, lower=None, upper=None, bins='auto'):
    """Return a pyplot histogram, where the upper and/or lower outliers are
    binned together for a more even distribution plot.

    Arguments:
        data -- dataframe to calculate the median absolute deviatan for
        z_treshold -- number of standard deviations from the median to determine outliers
        lower -- lower threshold for binning lower outliers together
        upper -- upper threshold for binning higer outliers together
    """
    if (not lower) & (not upper):
        lower, upper = calculate_z_range(data, z_treshold)

    if not lower or lower < data.min():
        lower = data.min()
        lower_outliers = False
    else:
        lower_outliers = True

    if not upper or upper > data.max():
        upper = data.max()
        upper_outliers = False
    else:
        upper_outliers = True

    n, bins, patches = plt.hist(data, range=(lower, upper), bins=bins)
    # cm = plt.cm.get_cmap('RdYlBu_r')
    # cm = plt.cm.get_cmap('Reds')
    # cm = plt.cm.get_cmap('RdPu')
    cm = plt.cm.get_cmap('magma')

    # To normalize your values
    col = (n - n.min()) / (n.max() - n.min())
    for c, p in zip(col, patches):
        # plt.setp(p, 'facecolor', cm(c))
        plt.setp(p, 'facecolor', cm(0.15 + 0.6 * c))

    if lower_outliers:
        n_lower_outliers = (data < lower).sum()
        patches[0].set_height(patches[0].get_height() + n_lower_outliers)
        patches[0].set_facecolor('b')
        patches[0].set_label('Lower outliers: ({:.2f}, {:.2f})'.format(data.min(), lower))

    if upper_outliers:
        n_upper_outliers = (data > upper).sum()
        patches[-1].set_height(patches[-1].get_height() + n_upper_outliers)
        patches[-1].set_facecolor('r')
        patches[-1].set_label('Upper outliers: ({:.2f}, {:.2f})'.format(upper, data.max()))

    if lower_outliers or upper_outliers:
        plt.legend()

    return plt


def mad(data):
    """Return the median absolute deviation for the data set.

    Arguments:
        data -- the dataframe to calculate the median absolute deviation for
    """
    median = np.median(data)
    diff = np.abs(data - median)
    mad = np.median(diff)
    return mad


def calculate_z_range(data, z_thresh=3.5):
    """Calculate the range for the dataset base on the specified Z threshold: the number of standard
    deviations from the median.

    Arguments:
        data -- the dataframe to calculate the median absolute deviation for.
        z_thresh -- the Z threshold.
    """
    MAD = mad(data)
    median = np.median(data)
    const = z_thresh * MAD / 0.6745  # sd = 0.6745*MAD
    return (median - const, median + const)